package net.tests.pieroxy.tools.j2ee;

import java.util.HashMap;
import java.util.Map;

import javax.servlet.FilterChain;
import javax.servlet.http.HttpServletRequest;

import junit.framework.TestCase;

import net.pieroxy.tools.j2ee.RedirectFilter;

public class RedirectFilterTest extends TestCase {
	
	public static void main(String[]args) throws Exception {
		System.out.println("tests running.");
		testsConfigDefaut();
		testsConfigSubstring();
		perfTest();
		testsConfigStatus();
		testsConfigQS();
		testsConfigAll();
		System.out.println("tests done.");
	}
	
  public static void testConfig() throws Exception {
    Map<String,String> params = new HashMap<String,String>();
    
    RedirectFilter f;
    
    params.clear();
    params.put("/index.html$","/");
    params.put("RedirectFilterConfig:patternType","substring");
    params.put("RedirectFilterConfig:status","3");
    params.put("RedirectFilterConfig:verboseLevel","5");
    params.put("RedirectFilterConfig:addQueryString","true");

    
    f = new RedirectFilter();
    assertEquals(f.isConfigAddQueryString(),false);
    assertEquals(f.getConfigStatus(),301);
    assertEquals(f.isMatchRegexp(),true);
    assertEquals(f.getVerboseLevel(),0);
    f.init(new FakeFilterConfig(params));
    assertEquals(f.isConfigAddQueryString(),true);
    assertEquals(f.getConfigStatus(),3);
    assertEquals(f.isMatchRegexp(),false);
    assertEquals(f.getVerboseLevel(),5);
    
    
  }
  
	public static void testsConfigDefaut() throws Exception {
		Map<String,String> params = new HashMap<String,String>();
		params.put("toto","titi");
		
		RedirectFilter f = new RedirectFilter();
		f.init(new FakeFilterConfig(params));
		
		FakeHttpServletResponse response = new FakeHttpServletResponse();
		f.doFilter(new FakeHttpServletRequest("http://www.toto.com/","blah=2&blue=3"),response, new FakeFilterChain());
		assertEquals(response.status, 301);
    assertEquals(response.location,"http://www.titi.com/");
		
		f.doFilter(new FakeHttpServletRequest("http://www.toto.com/toto.html",""),response = new FakeHttpServletResponse(), new FakeFilterChain());
    assertEquals(response.status, 301);
    assertEquals(response.location,"http://www.titi.com/titi.html");
		
		params.clear();
		params.put("/index.html$","/");
		f = new RedirectFilter();
		f.init(new FakeFilterConfig(params));
		f.doFilter(new FakeHttpServletRequest("http://www.toto.com/index.html.toto",""),response = new FakeHttpServletResponse(), new FakeFilterChain());
    assertEquals(response.status, 0);
    assertEquals(response.location,null);
		f.doFilter(new FakeHttpServletRequest("http://www.toto.com/mmm/index.html",""),response = new FakeHttpServletResponse(), new FakeFilterChain());
    assertEquals(response.status, 301);
    assertEquals(response.location,"http://www.toto.com/mmm/");
		f.doFilter(new FakeHttpServletRequest("http://www.toto.com/mmm/index_html",""),response = new FakeHttpServletResponse(), new FakeFilterChain());
    assertEquals(response.status, 301);
    assertEquals(response.location,"http://www.toto.com/mmm/");
	}
	
	public static void testsConfigSubstring() throws Exception {
		Map<String,String> params = new HashMap<String,String>();
		
		RedirectFilter f;
		FakeHttpServletResponse response;
		
		params.clear();
		params.put("/index.html$","/");
		params.put("RedirectFilterConfig:patternType","substring");
		f = new RedirectFilter();
		f.init(new FakeFilterConfig(params));
		f.doFilter(new FakeHttpServletRequest("http://www.toto.com/index.html.toto",""),response = new FakeHttpServletResponse(), new FakeFilterChain());
    assertEquals(response.status, 0);
    assertEquals(response.location,null);
		f.doFilter(new FakeHttpServletRequest("http://www.toto.com/mmm/index.html$---",""),response = new FakeHttpServletResponse(), new FakeFilterChain());
    assertEquals(response.status, 301);
    assertEquals(response.location,"http://www.toto.com/mmm/---");
		f.doFilter(new FakeHttpServletRequest("http://www.toto.com/mmm/index_html$",""),response = new FakeHttpServletResponse(), new FakeFilterChain());
    assertEquals(response.status, 0);
    assertEquals(response.location,null);
	}
	public static void testsConfigStatus() throws Exception {
		Map<String,String> params = new HashMap<String,String>();
		
		RedirectFilter f;
		FakeHttpServletResponse response;
		
		params.clear();
		params.put("/index.html$","/titi.html");
		params.put("RedirectFilterConfig:status","302");
		f = new RedirectFilter();
		f.init(new FakeFilterConfig(params));
		f.doFilter(new FakeHttpServletRequest("http://www.toto.com/index.html","blah"),response = new FakeHttpServletResponse(), new FakeFilterChain());
    assertEquals(response.status, 302);
    assertEquals(response.location,"http://www.toto.com/titi.html");
		
		params.clear();
		params.put("/index.html$","/titi.html");
		params.put("RedirectFilterConfig:status","404");
		f = new RedirectFilter();
		f.init(new FakeFilterConfig(params));
		f.doFilter(new FakeHttpServletRequest("http://www.toto.com/index.html",""),response = new FakeHttpServletResponse(), new FakeFilterChain());
    assertEquals(response.status, 404);
    assertEquals(response.location,null);
	}
	public static void testsConfigQS() throws Exception {
		Map<String,String> params = new HashMap<String,String>();
		
		RedirectFilter f;
		FakeHttpServletResponse response;
		
		params.clear();
		params.put("/index.html$","/titi.html");
		params.put("RedirectFilterConfig:addQueryString","true");
		params.put("RedirectFilterConfig:status","302");
		f = new RedirectFilter();
		f.init(new FakeFilterConfig(params));
		f.doFilter(new FakeHttpServletRequest("http://www.toto.com/index.html","h=1&j=2"),response = new FakeHttpServletResponse(), new FakeFilterChain());
    assertEquals(response.status, 302);
    assertEquals(response.location,"http://www.toto.com/titi.html?h=1&j=2");
	}
	public static void testsConfigAll() throws Exception {
		Map<String,String> params = new HashMap<String,String>();
		
		RedirectFilter f;
		FakeHttpServletResponse response;
		
		params.clear();
		params.put("/menu.one/","/newmenu/");
		params.put("RedirectFilterConfig:addQueryString","true");
		params.put("RedirectFilterConfig:patternType","substring");
		params.put("RedirectFilterConfig:status","302");
		f = new RedirectFilter();
		f.init(new FakeFilterConfig(params));
		f.doFilter(new FakeHttpServletRequest("http://www.toto.com/menu.one/index.html","h=1&j=2"),response = new FakeHttpServletResponse(), new FakeFilterChain());
    assertEquals(response.status, 302);
    assertEquals(response.location,"http://www.toto.com/newmenu/index.html?h=1&j=2");
		f.doFilter(new FakeHttpServletRequest("http://www.toto.com/menu_one/index.html","h=1&j=2"),response = new FakeHttpServletResponse(), new FakeFilterChain());
    assertEquals(response.status, 0);
    assertEquals(response.location,null);
		
		params.clear();
		params.put("/menu.one/$","/newmenu/");
		params.put("RedirectFilterConfig:addQueryString","true");
		params.put("RedirectFilterConfig:patternType","regexp");
		params.put("RedirectFilterConfig:status","302");
		f = new RedirectFilter();
		f.init(new FakeFilterConfig(params));
		f.doFilter(new FakeHttpServletRequest("http://www.toto.com/menu_one/","h=1&j=2"),response = new FakeHttpServletResponse(), new FakeFilterChain());
    assertEquals(response.status, 302);
    assertEquals(response.location,"http://www.toto.com/newmenu/?h=1&j=2");
		f.doFilter(new FakeHttpServletRequest("http://www.toto.com/menu_one/index.html","h=1&j=2"),response = new FakeHttpServletResponse(), new FakeFilterChain());
    assertEquals(response.status, 0);
    assertEquals(response.location,null);
	}
	
	public static void perfTest() throws Exception {
		
		Map<String,String> params = new HashMap<String,String>();
		
		RedirectFilter f;
		FakeHttpServletResponse response=new FakeHttpServletResponse();
		FilterChain fc = new FakeFilterChain();
		HttpServletRequest req = new FakeHttpServletRequest("http://www.toto.com/index.html.toto","");
		
		params.clear();
		params.put("/index.html$","/");
		params.put("/toto.","/");
		params.put("/toto1.","/");
		params.put("/toto2.","/");
		params.put("/toto3.","/");
		params.put("/toto4.","/");
		params.put("RedirectFilterConfig:patternType","substring");
		f = new RedirectFilter();
		f.init(new FakeFilterConfig(params));
		System.gc();
		long t = System.currentTimeMillis();
		for (int i=0 ; i<1000000 ; i++) f.doFilter(req,response, fc);
		System.out.println("ss:" + (System.currentTimeMillis()-t));
		
		params.clear();
		params.put("/index.html$","/");
		params.put("/toto.","/");
		params.put("/toto1.","/");
		params.put("/toto2.","/");
		params.put("/toto3.","/");
		params.put("/toto4.","/");
		f = new RedirectFilter();
		f.init(new FakeFilterConfig(params));
		System.gc();
		t = System.currentTimeMillis();
		for (int i=0 ; i<1000000 ; i++) f.doFilter(req,response, fc);
		System.out.println("re:" + (System.currentTimeMillis()-t));
	}


/*	public int countTestCases() {
		return 5;
	}


	public void run(TestResult arg0) {
		// TODO Auto-generated method stub
		try {
			testsConfigDefaut();
			testsConfigSubstring();
			testsConfigStatus();
			testsConfigQS();
			testsConfigAll();
			
		} catch (Exception e) {
			
		}
	}*/
}
